<?php

/**
 * @encoding     UTF-8
 * @package      WPObjects
 * @link         https://github.com/VladislavDolgolenko/WPObjects
 * @copyright    Copyright (C) 2018 Vladislav Dolgolenko
 * @license      MIT License
 * @author       Vladislav Dolgolenko <vladislavdolgolenko.com>
 * @support      <help@vladislavdolgolenko.com>
 */

namespace DM\WPObjects\Session;

class Storage implements 
    \DM\WPObjects\Service\ManagerInterface,
    \DM\WPObjects\Service\NamespaceInterface,
    \DM\WPObjects\EventManager\ListenerInterface,
    \DM\WPObjects\Session\CookiesInterface
{
    protected $namespace = '';
    
    protected $data = array();
    
    /**
     * Global service manager
     * 
     * @var \DM\WPObjects\Service\Manager
     */
    protected $ServiceManager = null;
    
    /**
     * @var \DM\WPObjects\Session\Cookies
     */
    protected $Cookies = null;
    
    public function attach()
    {
        if (\did_action('init')) {
            $this->initSession();
            $this->initStorage();
        } else {
            \add_action('init', array($this, 'initSession'), 10);
            \add_action('init', array($this, 'initStorage'), 11);
        }
    }
    
    public function detach()
    {
        \remove_action('init', array($this, 'initSession'), 10);
        \remove_action('init', array($this, 'initStorage'), 11);
    }
    
    public function initStorage()
    {
        $storage_data = $this->getStoragesData();
        if (isset($storage_data[$this->getSessionId()])) {
            $this->data = $storage_data[$this->getSessionId()];
        }
        
        return $this;
    }
    
    public function initSession()
    {
        $session_id = $this->getCookies()->get('session_id', null);
        if (!$session_id) {
            $session_id = $this->regenerateSessionId();
            $this->getCookies()->set('session_id', $session_id);
        }
        
        return $this;
    }
    
    protected function regenerateSessionId()
    {
        $bytes = openssl_random_pseudo_bytes(8);
        $session_id = bin2hex($bytes);
        $storages_data = $this->getStoragesData();
        if (isset($storages_data[$session_id])) {
            return $this->regenerateSessionId();
        }
        
        return $session_id;
    }
    
    public function getSessionId()
    {
        return $this->getCookies()->get('session_id', null);
    }
    
    public function set($key, $value)
    {
        $this->data[$key] = $value;
        
        return $this;
    }
    
    public function get($key, $default = null)
    {
        return isset($this->data[$key]) ? $this->data[$key] : $default;
    }
    
    public function update()
    {
        $data = $this->getData();
        $storages_data = $this->getStoragesData();
        
        $storages_data[$this->getSessionId()] = $data;
        $this->setStoragesData($storages_data);
        
        return $this;
    }
    
    public function getStoragesData()
    {
        $storages_data = \get_option($this->getStorageKey(), array());
        if (!is_array($storages_data)) {
            $storages_data = array();
        }
        
        return $storages_data;
    }
    
    public function setStoragesData($data)
    {
        \update_option($this->getStorageKey(), $data);
        
        return $this;
    }
    
    protected function getStorageKey()
    {
        return $this->getNamespace() . '_session_storage';
    } 
    
    /**
     * Return current session data
     * 
     * @return array
     */
    public function getData()
    {
        $data = $this->data;
        if (!is_array($data)) {
            $data = array();
        }
        
        return $data;
    }
    
    protected function setData($array)
    {
        $this->data = $array;
        
        return $this;
    }
    
    public function setNamespace($string)
    {
        $this->namespace = $string;
        
        return $this;
    }
    
    public function getNamespace()
    {
        return $this->namespace;
    }
    
    public function setServiceManager(\DM\WPObjects\Service\Manager $ServiceManager)
    {
        $this->ServiceManager = $ServiceManager;
        
        return $this;
    }
    
    /**
     * @return \DM\WPObjects\Service\Manager
     * @throws \Exception
     */
    public function getServiceManager()
    {
        if (is_null($this->ServiceManager)) {
            throw new \Exception('Undefined service manager');
        }
        
        return $this->ServiceManager;
    }
    
    public function setCookies(\DM\WPObjects\Session\Cookies $Cookies)
    {
        $this->Cookies = $Cookies;
        
        return $this;
    }
    
    /**
     * @return \DM\WPObjects\Session\Cookies
     */
    public function getCookies()
    {
        if (is_null($this->Cookies)) {
            throw new \Exception('Undefined \DM\WPObjects\Session\Cookies object');
        }
        
        return $this->Cookies;
    }
}